/* Copyright 2018 Kevin Ryde

   This file is part of nautyextra.

   nautyextra is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   nautyextra is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along
   with nautyextra.  See the file COPYING.  If not, see
   <http://www.gnu.org/licenses/>.
*/

/* Search all trees using gentreeg for the most maximum matchings.

   Results are the trees of Clemens Heuberger and Stephan Wagner, "The
   Number of Maximum Matchings In a Tree", Discrete Mathematics, volume 311,
   issue 21, November 2011, pages 2512-2542.
   https://arxiv.org/abs/1011.6554
*/

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <unistd.h>
#include <nauty/nauty.h>
#include <nauty/nausparse.h>
#include <nauty/naututil.h>
#include <nauty/gtools.h>
#include <nauty/gutils.h>

#include "nautyextra.h"
#include "nautyextra-wip.h"

#define WANT_DEBUG  0  /* set to 1 for debug prints */
#if WANT_DEBUG
#define DEBUG(expr)  expr
#else
#define DEBUG(expr)
#endif

int my_geng_main(int argc, char**argv);
int my_gentreeg_main(int argc, char**argv);

const int option_verbose = 1;

unsigned long long  trees_total_domnum[64];

#define MAX(x,y)  ((x) > (y) ? (x) : (y))
#define MIN(x,y)  ((x) < (y) ? (x) : (y))


unsigned long long  tree_count = 0;
unsigned long  max_value = -1;
int            max_reps = 0;
#define  MAX_STRS_SIZE  10
char *max_strs[MAX_STRS_SIZE];
int  max_vpars[MAX_STRS_SIZE][MAXN];

void
my_gentreeg_outproc (FILE *outfile, int *vpar, int n)
{
  DEBUG(printf("my_gentreeg_outproc() n=%d\n", n));
  tree_count++;

  struct nautyextra_matchnum_and_count info;
  nautyextra_matchnum_and_count_vpar(vpar,n, &info);

  if (info.count > max_value) {
    max_value = info.count;
    max_reps = 0;
    /* printf("new high %lu\n", info.count); */
    /* nautyextra_print_vpar(vpar,n); */
  }

  if (info.count == max_value) {
    if (max_reps < MAX_STRS_SIZE) {
      if (max_strs[max_reps]) free(max_strs[max_reps]);
      max_strs[max_reps] = strdup(nautyextra_vpar_to_s6(vpar,n));
      memcpy(max_vpars[max_reps], vpar, (n+1)*sizeof(int));
    }
    max_reps++;
  }
}

int
main (void)
{
  setbuf(stdout,NULL);
  nautyextra_want_progress = 1;

  int n;
  for (n = 11;
       n <= 15;
       n ++) {
    tree_count = 0;
    max_value = 0;
    max_reps = 0;
    time_t start_time;
    time(&start_time);

    if (0) {
      printf("gentreeg  n=%d\n", n);
    }
    char n_str[20];
    sprintf(n_str, "%d", n);

    int gentreeg_argc = 0;
    char *gentreeg_argv[10];
    gentreeg_argv[gentreeg_argc++] = "gentreeg"; /* progname */
    gentreeg_argv[gentreeg_argc++] = "-q";       /* quiet */
    gentreeg_argv[gentreeg_argc++] = n_str;
    gentreeg_argv[gentreeg_argc] = NULL;
    my_gentreeg_main(gentreeg_argc,gentreeg_argv);

    time_t end_time;
    time(&end_time);

    if (1) {
      printf("n=%d end, tree count %Lu, most matchings %lu reps %d\n",
             n, tree_count, max_value, max_reps);

      time_t elapsed_time = end_time - start_time;
      printf("  elapsed time %ld = %ld:%02ld\n",
             elapsed_time, elapsed_time/60, elapsed_time%60);
      int i;
      for (i = 0; i < max_reps; i++) {
        fputs(max_strs[i],stdout);
      }
      printf("\n");
    }
    if (1) {
      printf("%lu,\n", max_value);
    }
    if (0) {
      int i;
      for (i = 0; i < max_reps; i++) {
        nautyextra_print_vpar_gp(max_vpars[i],n);
        printf(",\n");
      }
    }
  }

  return 0;
}
