/* Copyright 2019 Kevin Ryde

   This file is part of nautyextra.

   nautyextra is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   nautyextra is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along
   with nautyextra.  See the file COPYING.  If not, see
   <http://www.gnu.org/licenses/>.
*/

/* Search all trees using gentreeg for periphery == all leaves.

*/

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include <nauty/nauty.h>
#include <nauty/nausparse.h>
#include <nauty/naututil.h>
#include <nauty/gtools.h>
#include <nauty/gutils.h>

#include "nautyextra.h"
#include "nautyextra-wip.h"

#define WANT_DEBUG  0  /* set to 1 for debug prints */
#if WANT_DEBUG
#define DEBUG(expr)  expr
#else
#define DEBUG(expr)
#endif

int my_gentreeg_main(int argc, char**argv);

const int option_verbose = 0;

unsigned long long  trees_total_domnum[64];

#define MAX(x,y)  ((x) > (y) ? (x) : (y))
#define MIN(x,y)  ((x) < (y) ? (x) : (y))


unsigned long long  count_found = 0;
unsigned long long  max_value = 0;
int                 max_reps = 0;
SG_DECL(sg);
SG_DECL(canon_sg);

void
my_gentreeg_outproc (FILE *outfile, int *vpar, int n)
{
  if (option_verbose) {
    printf("tree %p  n=%d\n", vpar,n);
    nautyextra_print_vpar(vpar,n);
  }

  int degrees[MAXN+1];
  nautyextra_degrees_array_vpar(vpar,n, degrees);

  int eccs[MAXN+1];
  nautyextra_eccentricities_vpar(vpar,n, eccs);
  int max_ecc = -1;
  int v;
  for (v=1; v<=n; v++) {
    if (degrees[v] > 1) { continue; }
    if (max_ecc == -1) { max_ecc = eccs[v]; }
    else if (eccs[v] != max_ecc) { return; }
  }
  count_found++;
}

int
main (void)
{
  setbuf(stdout,NULL);
  nautyextra_view_synchronous = 1;
  nautyextra_want_progress = 1;

  SG_INIT(sg);
  SG_INIT(canon_sg);
  int num_vertices;
  for (num_vertices = 2;
       num_vertices <= 40;
       num_vertices += 1) {
    /* printf("gentreeg  num_vertices=%d\n", num_vertices); */
    char num_vertices_str[20];
    sprintf(num_vertices_str, "%d", num_vertices);

    count_found = 0;
    max_value = 0;
    max_reps = 0;

    int gentreeg_argc = 0;
    char *gentreeg_argv[10];
    gentreeg_argv[gentreeg_argc++] = "gentreeg"; /* progname */
    gentreeg_argv[gentreeg_argc++] = "-q";       /* quiet */
    gentreeg_argv[gentreeg_argc++] = num_vertices_str;
    gentreeg_argv[gentreeg_argc] = NULL;
    my_gentreeg_main(gentreeg_argc,gentreeg_argv);

    printf("%Lu,", count_found);

    /* printf("n=%d  %Lu trees  max disjoint_domnum count %Lu reps %d \n", */
    /*      num_vertices, count_found, max_value, max_reps); */
  }

  return 0;
}
